
/*  A Bison parser, made from /home/gray/gnu/radius/lib/./users.y
    by GNU Bison version 1.28  */

#define YYBISON 1  /* Identify Bison output.  */

#define	EQ	257
#define	LT	258
#define	GT	259
#define	NE	260
#define	LE	261
#define	GE	262
#define	NUL	263
#define	BOGUS	264
#define	STRING	265
#define	QUOTE	266

#line 1 "users.y"

/* This file is part of GNU Radius.
   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.

   Written by Sergey Poznyakoff
  
   GNU Radius is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   GNU Radius is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with GNU Radius; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#if defined(HAVE_CONFIG_H)
# include <config.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <netinet/in.h>
         
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <sys/wait.h>

#include <common.h>

#define YYMAXDEPTH 10

static grad_locus_t start_loc;

static void *closure;
static register_rule_fp add_entry;
static grad_avp_t *grad_create_pair0(char *name, int op, char *valstr);

int uyyerror(char *s);
extern int uyylex();
 

#line 67 "users.y"
typedef union {
        char *string;
        grad_matching_rule_t *rule;
        struct {
                grad_avp_t *lhs, *rhs;
        } descr;
        grad_avp_t *pair;
        enum grad_operator op;
} YYSTYPE;
#ifndef YYDEBUG
#define YYDEBUG 1
#endif

#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		30
#define	YYFLAG		-32768
#define	YYNTBASE	14

#define YYTRANSLATE(x) ((unsigned)(x) <= 266 ? uyytranslate[x] : 24)

static const char uyytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,    13,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
     7,     8,     9,    10,    11,    12
};

#if YYDEBUG != 0
static const short uyyprhs[] = {     0,
     0,     1,     3,     5,     8,    11,    14,    17,    19,    22,
    24,    26,    28,    32,    36,    40,    42,    44,    46,    48,
    50,    52,    54
};

static const short uyyrhs[] = {    -1,
    15,     0,    16,     0,    15,    16,     0,    15,     1,     0,
    17,    18,     0,    17,     1,     0,    23,     0,    19,    19,
     0,     9,     0,    20,     0,    21,     0,    20,    13,    21,
     0,    11,    22,    23,     0,    11,    22,    10,     0,     3,
     0,     4,     0,     5,     0,     6,     0,     7,     0,     8,
     0,    11,     0,    12,     0
};

#endif

#if YYDEBUG != 0
static const short uyyrline[] = { 0,
    79,    80,    83,    86,    87,    93,    97,   109,   115,   122,
   126,   129,   130,   141,   145,   151,   155,   159,   163,   167,
   171,   177,   178
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const uyytname[] = {   "$","error","$undefined.","EQ","LT",
"GT","NE","LE","GE","NUL","BOGUS","STRING","QUOTE","','","input","list","entry",
"user","descr","npairlist","pairlist","pair","op","value", NULL
};
#endif

static const short uyyr1[] = {     0,
    14,    14,    15,    15,    15,    16,    16,    17,    18,    19,
    19,    20,    20,    21,    21,    22,    22,    22,    22,    22,
    22,    23,    23
};

static const short uyyr2[] = {     0,
     0,     1,     1,     2,     2,     2,     2,     1,     2,     1,
     1,     1,     3,     3,     3,     1,     1,     1,     1,     1,
     1,     1,     1
};

static const short uyydefact[] = {     1,
    22,    23,     0,     3,     0,     8,     5,     4,     7,    10,
     0,     6,     0,    11,    12,    16,    17,    18,    19,    20,
    21,     0,     9,     0,    15,    14,    13,     0,     0,     0
};

static const short uyydefgoto[] = {    28,
     3,     4,     5,    12,    13,    14,    15,    22,     6
};

static const short uyypact[] = {    -2,
-32768,-32768,     0,-32768,     4,-32768,-32768,-32768,-32768,-32768,
    13,-32768,    -3,    -6,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,    -8,-32768,     3,-32768,-32768,-32768,    22,    23,-32768
};

static const short uyypgoto[] = {-32768,
-32768,    21,-32768,-32768,    12,-32768,     2,-32768,     5
};


#define	YYLAST		27


static const short uyytable[] = {    -2,
     7,    25,     1,     2,     9,    10,    24,    11,     1,     2,
     1,     2,    10,    11,    11,    16,    17,    18,    19,    20,
    21,    29,    30,     8,    23,    27,    26
};

static const short uyycheck[] = {     0,
     1,    10,    11,    12,     1,     9,    13,    11,    11,    12,
    11,    12,     9,    11,    11,     3,     4,     5,     6,     7,
     8,     0,     0,     3,    13,    24,    22
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/share/bison.simple"
/* This file comes from bison-1.28.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

#ifndef YYSTACK_USE_ALLOCA
#ifdef alloca
#define YYSTACK_USE_ALLOCA
#else /* alloca not defined */
#ifdef __GNUC__
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
#define YYSTACK_USE_ALLOCA
#include <alloca.h>
#else /* not sparc */
/* We think this test detects Watcom and Microsoft C.  */
/* This used to test MSDOS, but that is a bad idea
   since that symbol is in the user namespace.  */
#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
#if 0 /* No need for malloc.h, which pollutes the namespace;
	 instead, just don't use alloca.  */
#include <malloc.h>
#endif
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
/* I don't know what this was needed for, but it pollutes the namespace.
   So I turned it off.   rms, 2 May 1997.  */
/* #include <malloc.h>  */
 #pragma alloca
#define YYSTACK_USE_ALLOCA
#else /* not MSDOS, or __TURBOC__, or _AIX */
#if 0
#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
		 and on HPUX 10.  Eventually we can turn this on.  */
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#endif /* __hpux */
#endif
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc */
#endif /* not GNU C */
#endif /* alloca not defined */
#endif /* YYSTACK_USE_ALLOCA not defined */

#ifdef YYSTACK_USE_ALLOCA
#define YYSTACK_ALLOC alloca
#else
#define YYSTACK_ALLOC malloc
#endif

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define uyyerrok		(uyyerrstatus = 0)
#define uyyclearin	(uyychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto uyyacceptlab
#define YYABORT 	goto uyyabortlab
#define YYERROR		goto uyyerrlab1
/* Like YYERROR except do call uyyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto uyyerrlab
#define YYRECOVERING()  (!!uyyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (uyychar == YYEMPTY && uyylen == 1)				\
    { uyychar = (token), uyylval = (value);			\
      uyychar1 = YYTRANSLATE (uyychar);				\
      YYPOPSTACK;						\
      goto uyybackup;						\
    }								\
  else								\
    { uyyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		uyylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		uyylex(&uyylval, &uyylloc, YYLEX_PARAM)
#else
#define YYLEX		uyylex(&uyylval, &uyylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		uyylex(&uyylval, YYLEX_PARAM)
#else
#define YYLEX		uyylex(&uyylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	uyychar;			/*  the lookahead symbol		*/
YYSTYPE	uyylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE uyylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int uyynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int uyydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Define __uyy_memcpy.  Note that the size argument
   should be passed with type unsigned int, because that is what the non-GCC
   definitions require.  With GCC, __builtin_memcpy takes an arg
   of type size_t, but it can handle unsigned int.  */

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __uyy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__uyy_memcpy (to, from, count)
     char *to;
     char *from;
     unsigned int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__uyy_memcpy (char *to, char *from, unsigned int count)
{
  register char *t = to;
  register char *f = from;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 217 "/usr/share/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into uyyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
#ifdef YYPARSE_PARAM
int uyyparse (void *);
#else
int uyyparse (void);
#endif
#endif

int
uyyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int uyystate;
  register int uyyn;
  register short *uyyssp;
  register YYSTYPE *uyyvsp;
  int uyyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int uyychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	uyyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE uyyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *uyyss = uyyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *uyyvs = uyyvsa;	/*  to allow uyyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE uyylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *uyyls = uyylsa;
  YYLTYPE *uyylsp;

#define YYPOPSTACK   (uyyvsp--, uyyssp--, uyylsp--)
#else
#define YYPOPSTACK   (uyyvsp--, uyyssp--)
#endif

  int uyystacksize = YYINITDEPTH;
  int uyyfree_stacks = 0;

#ifdef YYPURE
  int uyychar;
  YYSTYPE uyylval;
  int uyynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE uyylloc;
#endif
#endif

  YYSTYPE uyyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int uyylen;

#if YYDEBUG != 0
  if (uyydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  uyystate = 0;
  uyyerrstatus = 0;
  uyynerrs = 0;
  uyychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  uyyssp = uyyss - 1;
  uyyvsp = uyyvs;
#ifdef YYLSP_NEEDED
  uyylsp = uyyls;
#endif

/* Push a new state, which is found in  uyystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
uyynewstate:

  *++uyyssp = uyystate;

  if (uyyssp >= uyyss + uyystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *uyyvs1 = uyyvs;
      short *uyyss1 = uyyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *uyyls1 = uyyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = uyyssp - uyyss + 1;

#ifdef uyyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if uyyoverflow is a macro.  */
      uyyoverflow("parser stack overflow",
		 &uyyss1, size * sizeof (*uyyssp),
		 &uyyvs1, size * sizeof (*uyyvsp),
		 &uyyls1, size * sizeof (*uyylsp),
		 &uyystacksize);
#else
      uyyoverflow("parser stack overflow",
		 &uyyss1, size * sizeof (*uyyssp),
		 &uyyvs1, size * sizeof (*uyyvsp),
		 &uyystacksize);
#endif

      uyyss = uyyss1; uyyvs = uyyvs1;
#ifdef YYLSP_NEEDED
      uyyls = uyyls1;
#endif
#else /* no uyyoverflow */
      /* Extend the stack our own way.  */
      if (uyystacksize >= YYMAXDEPTH)
	{
	  uyyerror("parser stack overflow");
	  if (uyyfree_stacks)
	    {
	      free (uyyss);
	      free (uyyvs);
#ifdef YYLSP_NEEDED
	      free (uyyls);
#endif
	    }
	  return 2;
	}
      uyystacksize *= 2;
      if (uyystacksize > YYMAXDEPTH)
	uyystacksize = YYMAXDEPTH;
#ifndef YYSTACK_USE_ALLOCA
      uyyfree_stacks = 1;
#endif
      uyyss = (short *) YYSTACK_ALLOC (uyystacksize * sizeof (*uyyssp));
      __uyy_memcpy ((char *)uyyss, (char *)uyyss1,
		   size * (unsigned int) sizeof (*uyyssp));
      uyyvs = (YYSTYPE *) YYSTACK_ALLOC (uyystacksize * sizeof (*uyyvsp));
      __uyy_memcpy ((char *)uyyvs, (char *)uyyvs1,
		   size * (unsigned int) sizeof (*uyyvsp));
#ifdef YYLSP_NEEDED
      uyyls = (YYLTYPE *) YYSTACK_ALLOC (uyystacksize * sizeof (*uyylsp));
      __uyy_memcpy ((char *)uyyls, (char *)uyyls1,
		   size * (unsigned int) sizeof (*uyylsp));
#endif
#endif /* no uyyoverflow */

      uyyssp = uyyss + size - 1;
      uyyvsp = uyyvs + size - 1;
#ifdef YYLSP_NEEDED
      uyylsp = uyyls + size - 1;
#endif

#if YYDEBUG != 0
      if (uyydebug)
	fprintf(stderr, "Stack size increased to %d\n", uyystacksize);
#endif

      if (uyyssp >= uyyss + uyystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (uyydebug)
    fprintf(stderr, "Entering state %d\n", uyystate);
#endif

  goto uyybackup;
 uyybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* uyyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  uyyn = uyypact[uyystate];
  if (uyyn == YYFLAG)
    goto uyydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* uyychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (uyychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (uyydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      uyychar = YYLEX;
    }

  /* Convert token to internal form (in uyychar1) for indexing tables with */

  if (uyychar <= 0)		/* This means end of input. */
    {
      uyychar1 = 0;
      uyychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (uyydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      uyychar1 = YYTRANSLATE(uyychar);

#if YYDEBUG != 0
      if (uyydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", uyychar, uyytname[uyychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, uyychar, uyylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  uyyn += uyychar1;
  if (uyyn < 0 || uyyn > YYLAST || uyycheck[uyyn] != uyychar1)
    goto uyydefault;

  uyyn = uyytable[uyyn];

  /* uyyn is what to do for this token type in this state.
     Negative => reduce, -uyyn is rule number.
     Positive => shift, uyyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (uyyn < 0)
    {
      if (uyyn == YYFLAG)
	goto uyyerrlab;
      uyyn = -uyyn;
      goto uyyreduce;
    }
  else if (uyyn == 0)
    goto uyyerrlab;

  if (uyyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (uyydebug)
    fprintf(stderr, "Shifting token %d (%s), ", uyychar, uyytname[uyychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (uyychar != YYEOF)
    uyychar = YYEMPTY;

  *++uyyvsp = uyylval;
#ifdef YYLSP_NEEDED
  *++uyylsp = uyylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (uyyerrstatus) uyyerrstatus--;

  uyystate = uyyn;
  goto uyynewstate;

/* Do the default action for the current state.  */
uyydefault:

  uyyn = uyydefact[uyystate];
  if (uyyn == 0)
    goto uyyerrlab;

/* Do a reduction.  uyyn is the number of a rule to reduce with.  */
uyyreduce:
  uyylen = uyyr2[uyyn];
  if (uyylen > 0)
    uyyval = uyyvsp[1-uyylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (uyydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       uyyn, uyyrline[uyyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = uyyprhs[uyyn]; uyyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", uyytname[uyyrhs[i]]);
      fprintf (stderr, " -> %s\n", uyytname[uyyr1[uyyn]]);
    }
#endif


  switch (uyyn) {

case 3:
#line 84 "users.y"
{
           ;
    break;}
case 5:
#line 88 "users.y"
{
                   grad_parser_lex_sync(); uyyerrok; uyyclearin;
           ;
    break;}
case 6:
#line 94 "users.y"
{
                   add_entry(closure, &start_loc, uyyvsp[-1].string, uyyvsp[0].descr.lhs, uyyvsp[0].descr.rhs);
           ;
    break;}
case 7:
#line 98 "users.y"
{
                   grad_log(L_ERR, _("discarding user `%s'"), uyyvsp[-1].string);
                   if (grad_parser_lex_sync() <= 0)
			   uyychar = 0; /* force end-of-file */
		   else {
			   uyyerrok;
			   uyyclearin;
		   }
           ;
    break;}
case 8:
#line 110 "users.y"
{
		   start_loc = grad_parser_source_locus;
           ;
    break;}
case 9:
#line 116 "users.y"
{
                   uyyval.descr.lhs = uyyvsp[-1].pair;
                   uyyval.descr.rhs = uyyvsp[0].pair;
           ;
    break;}
case 10:
#line 123 "users.y"
{
                   uyyval.pair = NULL;
           ;
    break;}
case 13:
#line 131 "users.y"
{
                   if (uyyvsp[-2].pair) {
                           if (uyyvsp[0].pair) 
                                   grad_avl_add_list(&uyyvsp[-2].pair, uyyvsp[0].pair);
                           uyyval.pair = uyyvsp[-2].pair;
                   } else
                           uyyval.pair = uyyvsp[0].pair;
           ;
    break;}
case 14:
#line 142 "users.y"
{
                   uyyval.pair = grad_create_pair0(uyyvsp[-2].string, uyyvsp[-1].op, uyyvsp[0].string);   
           ;
    break;}
case 15:
#line 146 "users.y"
{
                   YYERROR;
           ;
    break;}
case 16:
#line 152 "users.y"
{
                   uyyval.op = grad_operator_equal;
           ;
    break;}
case 17:
#line 156 "users.y"
{
                   uyyval.op = grad_operator_less_than;
           ;
    break;}
case 18:
#line 160 "users.y"
{ 
                   uyyval.op = grad_operator_greater_than;
           ;
    break;}
case 19:
#line 164 "users.y"
{
                   uyyval.op = grad_operator_not_equal;
           ;
    break;}
case 20:
#line 168 "users.y"
{
                   uyyval.op = grad_operator_less_equal;
           ;
    break;}
case 21:
#line 172 "users.y"
{
                   uyyval.op = grad_operator_greater_equal;
           ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 543 "/usr/share/bison.simple"

  uyyvsp -= uyylen;
  uyyssp -= uyylen;
#ifdef YYLSP_NEEDED
  uyylsp -= uyylen;
#endif

#if YYDEBUG != 0
  if (uyydebug)
    {
      short *ssp1 = uyyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != uyyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++uyyvsp = uyyval;

#ifdef YYLSP_NEEDED
  uyylsp++;
  if (uyylen == 0)
    {
      uyylsp->first_line = uyylloc.first_line;
      uyylsp->first_column = uyylloc.first_column;
      uyylsp->last_line = (uyylsp-1)->last_line;
      uyylsp->last_column = (uyylsp-1)->last_column;
      uyylsp->text = 0;
    }
  else
    {
      uyylsp->last_line = (uyylsp+uyylen-1)->last_line;
      uyylsp->last_column = (uyylsp+uyylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  uyyn = uyyr1[uyyn];

  uyystate = uyypgoto[uyyn - YYNTBASE] + *uyyssp;
  if (uyystate >= 0 && uyystate <= YYLAST && uyycheck[uyystate] == *uyyssp)
    uyystate = uyytable[uyystate];
  else
    uyystate = uyydefgoto[uyyn - YYNTBASE];

  goto uyynewstate;

uyyerrlab:   /* here on detecting error */

  if (! uyyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++uyynerrs;

#ifdef YYERROR_VERBOSE
      uyyn = uyypact[uyystate];

      if (uyyn > YYFLAG && uyyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -uyyn if nec to avoid negative indexes in uyycheck.  */
	  for (x = (uyyn < 0 ? -uyyn : 0);
	       x < (sizeof(uyytname) / sizeof(char *)); x++)
	    if (uyycheck[x + uyyn] == x)
	      size += strlen(uyytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (uyyn < 0 ? -uyyn : 0);
		       x < (sizeof(uyytname) / sizeof(char *)); x++)
		    if (uyycheck[x + uyyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, uyytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      uyyerror(msg);
	      free(msg);
	    }
	  else
	    uyyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	uyyerror("parse error");
    }

  goto uyyerrlab1;
uyyerrlab1:   /* here on error raised explicitly by an action */

  if (uyyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (uyychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (uyydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", uyychar, uyytname[uyychar1]);
#endif

      uyychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  uyyerrstatus = 3;		/* Each real token shifted decrements this */

  goto uyyerrhandle;

uyyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  uyyn = uyydefact[uyystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (uyyn) goto uyydefault;
#endif

uyyerrpop:   /* pop the current state because it cannot handle the error token */

  if (uyyssp == uyyss) YYABORT;
  uyyvsp--;
  uyystate = *--uyyssp;
#ifdef YYLSP_NEEDED
  uyylsp--;
#endif

#if YYDEBUG != 0
  if (uyydebug)
    {
      short *ssp1 = uyyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != uyyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

uyyerrhandle:

  uyyn = uyypact[uyystate];
  if (uyyn == YYFLAG)
    goto uyyerrdefault;

  uyyn += YYTERROR;
  if (uyyn < 0 || uyyn > YYLAST || uyycheck[uyyn] != YYTERROR)
    goto uyyerrdefault;

  uyyn = uyytable[uyyn];
  if (uyyn < 0)
    {
      if (uyyn == YYFLAG)
	goto uyyerrpop;
      uyyn = -uyyn;
      goto uyyreduce;
    }
  else if (uyyn == 0)
    goto uyyerrpop;

  if (uyyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (uyydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++uyyvsp = uyylval;
#ifdef YYLSP_NEEDED
  *++uyylsp = uyylloc;
#endif

  uyystate = uyyn;
  goto uyynewstate;

 uyyacceptlab:
  /* YYACCEPT comes here.  */
  if (uyyfree_stacks)
    {
      free (uyyss);
      free (uyyvs);
#ifdef YYLSP_NEEDED
      free (uyyls);
#endif
    }
  return 0;

 uyyabortlab:
  /* YYABORT comes here.  */
  if (uyyfree_stacks)
    {
      free (uyyss);
      free (uyyvs);
#ifdef YYLSP_NEEDED
      free (uyyls);
#endif
    }
  return 1;
}
#line 181 "users.y"


int
uyyerror(char *s)
{
	grad_log_loc(L_ERR, &grad_parser_source_locus, "%s", s);
	return 0;
}

grad_avp_t *
grad_create_pair0(char *name, int op, char *valstr)
{
	return grad_create_pair(&grad_parser_source_locus, name, op, valstr);
}

grad_avp_t *
grad_create_pair(grad_locus_t *loc, char *name, 
                 enum grad_operator op, char *valstr)
{
        grad_dict_attr_t *attr = NULL;
        grad_dict_value_t *dval;
        grad_avp_t *pair, *pair2;
        char *s;
        int x;
        time_t timeval;
        struct tm *tm, tms;
        
        if ((attr = grad_attr_name_to_dict(name)) == NULL) {
                grad_log_loc(L_ERR, loc, _("unknown attribute `%s'"),
			     name);
                return NULL;
        }

        pair = grad_avp_alloc();
        
        pair->next = NULL;
        pair->name = attr->name;
        pair->attribute = attr->value;
        pair->type = attr->type;
        pair->prop = attr->prop;
        pair->operator = op;

        if (valstr[0] == '=') {
                pair->eval_type = grad_eval_interpret;
                pair->avp_strvalue = grad_estrdup(valstr+1);
                pair->avp_strlength = strlen(pair->avp_strvalue);
                return pair;
        }

        pair->eval_type = grad_eval_const;
        
        switch (pair->type) {
        case GRAD_TYPE_STRING:
                if (pair->attribute == DA_EXEC_PROGRAM ||
                    pair->attribute == DA_EXEC_PROGRAM_WAIT) {
                        if (valstr[0] != '/' && valstr[0] != '|') {
                                grad_log_loc(L_ERR, loc,
					     _("%s: not an absolute pathname"),
					     name);
                                grad_avp_free(pair);
                                return NULL;
                        }
                }

		pair->avp_strvalue = grad_estrdup(valstr);
                pair->avp_strlength = strlen(pair->avp_strvalue);
		
		if (attr->parser && attr->parser(pair, &s)) {
			grad_log_loc(L_ERR, loc,
				     "%s %s: %s",
				     _("attribute"),
				     pair->name, s);
			free(s);
			grad_avp_free(pair);
			return NULL;
		}

                break;

        case GRAD_TYPE_INTEGER:
                /*
                 *      For DA_NAS_PORT_ID, allow a
                 *      port range instead of just a port.
                 */
                if (attr->value == DA_NAS_PORT_ID) {
                        for (s = valstr; *s; s++)
                                if (!isdigit(*s))
                                        break;
                        if (*s) {
                                pair->type = GRAD_TYPE_STRING;
                                pair->avp_strvalue = grad_estrdup(valstr);
                                pair->avp_strlength = strlen(pair->avp_strvalue);
                                break;
                        }
                }
                if (isdigit(*valstr)) {
                        pair->avp_lvalue = atoi(valstr);
                } else if ((dval = grad_value_name_to_value(valstr,
							    pair->attribute))
			         == NULL) {
                        grad_avp_free(pair);
                        grad_log_loc(L_ERR, loc,
				_("value %s is not declared for attribute %s"),
				     valstr, name);
                        return NULL;
                } else {
                        pair->avp_lvalue = dval->value;
                }
                break;

        case GRAD_TYPE_IPADDR:
                if (pair->attribute != DA_FRAMED_IP_ADDRESS) {
                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);
                } else {
                        /*
                         *      We allow a "+" at the end to
                         *      indicate that we should add the
                         *      portno. to the IP address.
                         */
                        x = 0;
                        if (valstr[0]) {
                                for(s = valstr; s[1]; s++)
                                        ;
                                if (*s == '+') {
                                        *s = 0;
                                        x = 1;
                                }
                        }
                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);

			if (x) {
				char *s;
				asprintf(&s, "%lu+%%{NAS-Port-Id}",
					 pair->avp_lvalue);
				pair->avp_strvalue = grad_estrdup(s);
				pair->avp_strlength = strlen(pair->avp_strvalue);
				pair->eval_type = grad_eval_interpret;
				free(s);
			}
                }
                break;
                
        case GRAD_TYPE_DATE:
                timeval = time(0);
                tm = localtime_r(&timeval, &tms);
                if (grad_parse_time_string(valstr, tm)) {
                        grad_log_loc(L_ERR, loc, _("%s: can't parse date"),
				     name);
                        grad_avp_free(pair);
                        return NULL;
                }
#ifdef TIMELOCAL
                pair->avp_lvalue = (grad_uint32_t)timelocal(tm);
#else /* TIMELOCAL */
                pair->avp_lvalue = (grad_uint32_t)mktime(tm);
#endif /* TIMELOCAL */
                break;

        default:
                grad_log_loc(L_ERR, loc,
			     _("%s: unknown attribute type %d"),
			     name, pair->type);
                grad_avp_free(pair);
                return NULL;
        }

        return pair;
}

extern int uyydebug;

int
grad_parse_rule_file(char *file, void *c, register_rule_fp f)
{
        int rc;
        
        if (grad_parser_lex_init(file))
                return -1;
        closure = c;
        add_entry = f;

        uyydebug = 0;
        rc = uyyparse();
        grad_parser_lex_finish();
        return rc;
}

void
grad_enable_rule_debug(int val)
{
        uyydebug = val;
	grad_log_loc(L_NOTICE, &grad_parser_source_locus,
		     uyydebug ? _("enabled userfile parser debugging") :
             	 	       _("disabled userfile parser debugging"));
}
